@@ -1,5 +1,5 @@
 /* auparse.h --
- * Copyright 2006-08,2012 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2006-08,2012,2014,2015 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -42,10 +42,12 @@ typedef void (*auparse_callback_ptr)(auparse_state_t *au,
 /* General functions that affect operation of the library */
 auparse_state_t *auparse_init(ausource_t source, const void *b);
 int auparse_feed(auparse_state_t *au, const char *data, size_t data_len);
+void auparse_feed_age_events(const auparse_state_t *au);
 int auparse_flush_feed(auparse_state_t *au);
 int auparse_feed_has_data(const auparse_state_t *au);
 void auparse_add_callback(auparse_state_t *au, auparse_callback_ptr callback,
 			void *user_data, user_destroy user_destroy_func);
+void auparse_set_escape_mode(auparse_esc_t mode);
 int auparse_reset(auparse_state_t *au);
 void auparse_destroy(auparse_state_t *au);
 
@@ -58,6 +60,8 @@ int ausearch_add_interpreted_item(auparse_state_t *au, const char *field,
 			const char *op, const char *value, ausearch_rule_t how);
 int ausearch_add_timestamp_item(auparse_state_t *au, const char *op, time_t sec,
 				unsigned milli, ausearch_rule_t how);
+int ausearch_add_timestamp_item_ex(auparse_state_t *au, const char *op,
+	time_t sec, unsigned milli, unsigned serial, ausearch_rule_t how);
 int ausearch_add_regex(auparse_state_t *au, const char *expr);
 int ausearch_set_stop(auparse_state_t *au, austop_t where);
 void ausearch_clear(auparse_state_t *au);
@@ -83,6 +87,7 @@ int auparse_goto_record_num(auparse_state_t *au, unsigned int num);
 
 /* Accessors to record data */
 int auparse_get_type(auparse_state_t *au);
+const char *auparse_get_type_name(auparse_state_t *au);
 unsigned int auparse_get_line_number(auparse_state_t *au);
 const char *auparse_get_filename(auparse_state_t *au);
 int auparse_first_field(auparse_state_t *au);
