@@ -110,6 +110,8 @@
 #include "pktoptnametabs.h"
 #include "umounttabs.h"
 #include "ioctlreqtabs.h"
+#include "inethooktabs.h"
+#include "netactiontabs.h"
 
 typedef enum { AVC_UNSET, AVC_DENIED, AVC_GRANTED } avc_t;
 typedef enum { S_UNSET=-1, S_FAILED, S_SUCCESS } success_t;
@@ -163,6 +165,7 @@ static void tty_escape(const char *s, char *dest, unsigned int len)
 			dest[j++] = s[i];
 		i++;
 	}
+	dest[j] = '\0';	/* terminate string */
 }
 
 static const char sh_set[] = "\"'`$\\";
@@ -195,6 +198,7 @@ static void shell_escape(const char *s, char *dest, unsigned int len)
 			dest[j++] = s[i];
 		i++;
 	}
+	dest[j] = '\0';	/* terminate string */
 }
 
 static const char quote_set[] = ";'\"`#$&*?[]<>{}\\";
@@ -227,6 +231,7 @@ static void shell_quote_escape(const char *s, char *dest, unsigned int len)
 			dest[j++] = s[i];
 		i++;
 	}
+	dest[j] = '\0';	/* terminate string */
 }
 
 /* This should return the count of what needs escaping */
@@ -268,7 +273,6 @@ int set_escape_mode(auparse_esc_t mode)
 	escape_mode = mode;
 	return 0;
 }
-hidden_def(set_escape_mode)
 
 static int is_hex_string(const char *str)
 {
@@ -601,7 +605,8 @@ static const char *print_exit(const char *val)
         }
 
         if (ival < 0) {
-		if (asprintf(&out, "%lld(%s)", ival, strerror(-ival)) < 0)
+		if (asprintf(&out, "%s(%s)", audit_errno_to_name(-ival),
+					strerror(-ival)) < 0)
 			out = NULL;
 		return out;
         }
@@ -1891,6 +1896,19 @@ static const char *print_ioctl_req(const char *val)
 	return out;
 }
 
+static const char *print_exit_syscall(const char *val)
+{
+	char *out;
+
+	if (strcmp(val, "0") == 0)
+		out = strdup("EXIT_SUCCESS");
+	else if (strcmp(val, "1") == 0)
+		out = strdup("EXIT_FAILURE");
+	else
+		out = strdup("UNKNOWN");
+	return out;
+}
+
 static const char *print_a0(const char *val, const idata *id)
 {
 	char *out;
@@ -1971,6 +1989,8 @@ static const char *print_a0(const char *val, const idata *id)
 			return print_dirfd(val);
                	else if (strcmp(sys, "ipccall") == 0)
 			return print_ipccall(val, 16);
+		else if (strncmp(sys, "exit", 4) == 0)
+			return print_exit_syscall(val);
 	}
 	if (asprintf(&out, "0x%s", val) < 0)
 			out = NULL;
@@ -2268,6 +2288,71 @@ static const char *print_protocol(const char *val)
 	return out;
 }
 
+/* FIXME - this assumes inet hook. Could also be an arp hook */
+static const char *print_hook(const char *val)
+{
+	int hook;
+	char *out;
+	const char *str;
+
+	errno = 0;
+	hook = strtoul(val, NULL, 16);
+	if (errno) {
+		if (asprintf(&out, "conversion error(%s)", val) < 0)
+			out = NULL;
+		return out;
+	}
+	str = inethook_i2s(hook);
+	if (str == NULL) {
+		if (asprintf(&out, "unknown hook(%s)", val) < 0)
+			out = NULL;
+		return out;
+	} else
+		return strdup(str);
+}
+
+static const char *print_netaction(const char *val)
+{
+	int action;
+	char *out;
+	const char *str;
+
+	errno = 0;
+	action = strtoul(val, NULL, 16);
+	if (errno) {
+		if (asprintf(&out, "conversion error(%s)", val) < 0)
+			out = NULL;
+		return out;
+	}
+	str = netaction_i2s(action);
+	if (str == NULL) {
+		if (asprintf(&out, "unknown action(%s)", val) < 0)
+			out = NULL;
+		return out;
+	} else
+		return strdup(str);
+}
+
+/* Ethernet packet types */
+static const char *print_macproto(const char *val)
+{
+	int type;
+	char *out;
+
+	errno = 0;
+	type = strtoul(val, NULL, 16);
+	if (errno) {
+		if (asprintf(&out, "conversion error(%s)", val) < 0)
+			out = NULL;
+		return out;
+	}
+	if (type == 0x0800)
+		return strdup("IP");
+	else if (type == 0x0806)
+		return strdup("ARP");
+	return strdup("UNKNOWN");
+}
+
 static const char *print_addr(const char *val)
 {
 	char *out = strdup(val);
@@ -2525,7 +2610,6 @@ int auparse_interp_adjust_type(int rtype, const char *name, const char *val)
 
 	return type;
 }
-hidden_def(auparse_interp_adjust_type)
 
 const char *auparse_do_interpretation(int type, const idata *id)
 {
@@ -2627,6 +2711,15 @@ const char *auparse_do_interpretation(int type, const idata *id)
 		case AUPARSE_TYPE_PROCTITLE:
 			out = print_proctitle(id->val);
 			break;
+		case AUPARSE_TYPE_HOOK:
+			out = print_hook(id->val);
+			break;
+		case AUPARSE_TYPE_NETACTION:
+			out = print_netaction(id->val);
+			break;
+		case AUPARSE_TYPE_MACPROTO:
+			out = print_macproto(id->val);
+			break;
 		case AUPARSE_TYPE_MAC_LABEL:
 		case AUPARSE_TYPE_UNCLASSIFIED:
 		default:
@@ -2647,5 +2740,4 @@ const char *auparse_do_interpretation(int type, const idata *id)
 	}
 	return out;
 }
-hidden_def(auparse_do_interpretation)
 
