@@ -1,5 +1,5 @@
 /* libaudit.c -- 
- * Copyright 2004-2009,2012,2014 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2009,2012,2014,2016 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -82,6 +82,7 @@ static const struct nv_list failure_actions[] =
 int _audit_permadded = 0;
 int _audit_archadded = 0;
 int _audit_syscalladded = 0;
+int _audit_exeadded = 0;
 unsigned int _audit_elf = 0U;
 static struct libaudit_conf config;
 
@@ -114,7 +115,6 @@ int audit_request_status(int fd)
 			"Error sending status request (%s)", strerror(-rc));
 	return rc;
 }
-hidden_def(audit_request_status)
 
 /*
  * Set everything to its default value
@@ -534,7 +534,6 @@ int audit_set_feature(int fd, unsigned feature, unsigned value, unsigned lock)
 	return -1;
 #endif
 }
-hidden_def(audit_set_feature)
 
 int audit_request_features(int fd)
 {
@@ -554,7 +553,6 @@ int audit_request_features(int fd)
 	return -1;
 #endif
 }
-hidden_def(audit_request_features)
 
 extern int  audit_set_loginuid_immutable(int fd)
 {
@@ -566,6 +564,66 @@ extern int  audit_set_loginuid_immutable(int fd)
 #endif
 }
 
+#define AUDIT_FEATURES_UNSET 0xFFFFFFFF
+#define AUDIT_FEATURES_UNSUPPORTED 0xEFFFFFFF
+static uint32_t features_bitmap = AUDIT_FEATURES_UNSET;
+static void load_feature_bitmap(void)
+{
+	int rc, fd;
+
+	fd = audit_open();
+	if (fd < 0) {
+		features_bitmap = AUDIT_FEATURES_UNSUPPORTED;
+		return;
+	}
+
+#if HAVE_DECL_AUDIT_FEATURE_VERSION
+	if ((rc = audit_request_status(fd)) > 0) {
+		struct audit_reply rep;
+		int i;
+		int timeout = 40; /* tenths of seconds */
+		struct pollfd pfd[1];
+
+		pfd[0].fd = fd;
+		pfd[0].events = POLLIN;
+
+	        for (i = 0; i < timeout; i++) {
+			do {
+				rc = poll(pfd, 1, 100);
+			} while (rc < 0 && errno == EINTR);
+
+			rc = audit_get_reply(fd, &rep, GET_REPLY_NONBLOCKING,0);
+			if (rc > 0) {
+                	        /* If we get done or error, break out */
+                        	if (rep.type == NLMSG_DONE || 
+					rep.type == NLMSG_ERROR)
+	                                break;
+
+        	                /* If its not status, keep looping */
+	                        if (rep.type != AUDIT_GET)
+        	                        continue;
+
+				/* Found it... */
+				features_bitmap = rep.status->feature_bitmap;
+				return;
+			}
+		}
+	}
+#endif
+	features_bitmap = AUDIT_FEATURES_UNSUPPORTED;
+}
+
+uint32_t audit_get_features(void)
+{
+	if (features_bitmap == AUDIT_FEATURES_UNSET)
+		load_feature_bitmap();
+
+	if (features_bitmap == AUDIT_FEATURES_UNSUPPORTED)
+		return 0;
+
+	return features_bitmap;
+}
+
 int audit_request_rules_list_data(int fd)
 {
 	int rc = audit_send(fd, AUDIT_LIST_RULES, NULL, 0);
@@ -668,7 +726,6 @@ int audit_add_watch_dir(int type, struct audit_rule_data **rulep,
 
 	return  0;
 }
-hidden_def(audit_add_watch_dir)
 
 int audit_add_rule_data(int fd, struct audit_rule_data *rule,
                         int flags, int action)
@@ -833,7 +890,6 @@ int audit_rule_syscall_data(struct audit_rule_data *rule, int scall)
 	rule->mask[word] |= bit;
 	return 0;
 }
-hidden_def(audit_rule_syscall_data)
 
 int audit_rule_syscallbyname_data(struct audit_rule_data *rule,
                                   const char *scall)
@@ -861,7 +917,6 @@ int audit_rule_syscallbyname_data(struct audit_rule_data *rule,
 		return audit_rule_syscall_data(rule, nr);
 	return -1;
 }
-hidden_def(audit_rule_syscallbyname_data)
 
 int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 					 const char *pair,
@@ -920,19 +975,24 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_EUID:
 			switch(field2) {
 			case AUDIT_LOGINUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_AUID_TO_EUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_AUID_TO_EUID;
 				break;
 			case AUDIT_FSUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EUID_TO_FSUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EUID_TO_FSUID;
 				break;
 			case AUDIT_OBJ_UID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EUID_TO_OBJ_UID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EUID_TO_OBJ_UID;
 				break;
 			case AUDIT_SUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EUID_TO_SUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EUID_TO_SUID;
 				break;
 			case AUDIT_UID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_UID_TO_EUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_UID_TO_EUID;
 				break;
 			default:
 				return -1;
@@ -941,19 +1001,24 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_FSUID:
 			switch(field2) {
 			case AUDIT_LOGINUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_AUID_TO_FSUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_AUID_TO_FSUID;
 				break;
 			case AUDIT_EUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EUID_TO_FSUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EUID_TO_FSUID;
 				break;
 			case AUDIT_OBJ_UID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_FSUID_TO_OBJ_UID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_FSUID_TO_OBJ_UID;
 				break;
 			case AUDIT_SUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_SUID_TO_FSUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_SUID_TO_FSUID;
 				break;
 			case AUDIT_UID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_UID_TO_FSUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_UID_TO_FSUID;
 				break;
 			default:
 				return -1;
@@ -962,19 +1027,24 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_LOGINUID:
 			switch(field2) {
 			case AUDIT_EUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_AUID_TO_EUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_AUID_TO_EUID;
 				break;
 			case AUDIT_FSUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_AUID_TO_FSUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_AUID_TO_FSUID;
 				break;
 			case AUDIT_OBJ_UID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_AUID_TO_OBJ_UID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_AUID_TO_OBJ_UID;
 				break;
 			case AUDIT_SUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_AUID_TO_SUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_AUID_TO_SUID;
 				break;
 			case AUDIT_UID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_UID_TO_AUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_UID_TO_AUID;
 				break;
 			default:
 				return -1;
@@ -983,19 +1053,24 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_SUID:
 			switch(field2) {
 			case AUDIT_LOGINUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_AUID_TO_SUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_AUID_TO_SUID;
 				break;
 			case AUDIT_EUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EUID_TO_SUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EUID_TO_SUID;
 				break;
 			case AUDIT_FSUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_SUID_TO_FSUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_SUID_TO_FSUID;
 				break;
 			case AUDIT_OBJ_UID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_SUID_TO_OBJ_UID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_SUID_TO_OBJ_UID;
 				break;
 			case AUDIT_UID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_UID_TO_SUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_UID_TO_SUID;
 				break;
 			default:
 				return -1;
@@ -1004,19 +1079,24 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_OBJ_UID:
 			switch(field2) {
 			case AUDIT_LOGINUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_AUID_TO_OBJ_UID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_AUID_TO_OBJ_UID;
 				break;
 			case AUDIT_EUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EUID_TO_OBJ_UID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EUID_TO_OBJ_UID;
 				break;
 			case AUDIT_FSUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_FSUID_TO_OBJ_UID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_FSUID_TO_OBJ_UID;
 				break;
 			case AUDIT_UID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_UID_TO_OBJ_UID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_UID_TO_OBJ_UID;
 				break;
 			case AUDIT_SUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_SUID_TO_OBJ_UID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_SUID_TO_OBJ_UID;
 				break;
 			default:
 				return -1;
@@ -1025,19 +1105,24 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_UID:
 			switch(field2) {
 			case AUDIT_LOGINUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_UID_TO_AUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_UID_TO_AUID;
 				break;
 			case AUDIT_EUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_UID_TO_EUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_UID_TO_EUID;
 				break;
 			case AUDIT_FSUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_UID_TO_FSUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_UID_TO_FSUID;
 				break;
 			case AUDIT_OBJ_UID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_UID_TO_OBJ_UID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_UID_TO_OBJ_UID;
 				break;
 			case AUDIT_SUID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_UID_TO_SUID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_UID_TO_SUID;
 				break;
 			default:
 				return -1;
@@ -1048,16 +1133,20 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_EGID:
 			switch(field2) {
 			case AUDIT_FSGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EGID_TO_FSGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EGID_TO_FSGID;
 				break;
 			case AUDIT_GID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_GID_TO_EGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_GID_TO_EGID;
 				break;
 			case AUDIT_OBJ_GID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EGID_TO_OBJ_GID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EGID_TO_OBJ_GID;
 				break;
 			case AUDIT_SGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EGID_TO_SGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EGID_TO_SGID;
 				break;
 			default:
 				return -1;
@@ -1066,16 +1155,20 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_FSGID:
 			switch(field2) {
 			case AUDIT_SGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_SGID_TO_FSGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_SGID_TO_FSGID;
 				break;
 			case AUDIT_GID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_GID_TO_FSGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_GID_TO_FSGID;
 				break;
 			case AUDIT_OBJ_GID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_FSGID_TO_OBJ_GID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_FSGID_TO_OBJ_GID;
 				break;
 			case AUDIT_EGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EGID_TO_FSGID;
+				rule->values[rule->field_count] =
+						 AUDIT_COMPARE_EGID_TO_FSGID;
 				break;
 			default:
 				return -1;
@@ -1084,16 +1177,20 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_GID:
 			switch(field2) {
 			case AUDIT_EGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_GID_TO_EGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_GID_TO_EGID;
 				break;
 			case AUDIT_FSGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_GID_TO_FSGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_GID_TO_FSGID;
 				break;
 			case AUDIT_OBJ_GID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_GID_TO_OBJ_GID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_GID_TO_OBJ_GID;
 				break;
 			case AUDIT_SGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_GID_TO_SGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_GID_TO_SGID;
 				break;
 			default:
 				return -1;
@@ -1102,16 +1199,20 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_OBJ_GID:
 			switch(field2) {
 			case AUDIT_EGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EGID_TO_OBJ_GID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EGID_TO_OBJ_GID;
 				break;
 			case AUDIT_FSGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_FSGID_TO_OBJ_GID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_FSGID_TO_OBJ_GID;
 				break;
 			case AUDIT_GID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_GID_TO_OBJ_GID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_GID_TO_OBJ_GID;
 				break;
 			case AUDIT_SGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_SGID_TO_OBJ_GID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_SGID_TO_OBJ_GID;
 				break;
 			default:
 				return -1;
@@ -1120,16 +1221,20 @@ int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 		case AUDIT_SGID:
 			switch(field2) {
 			case AUDIT_FSGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_SGID_TO_FSGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_SGID_TO_FSGID;
 				break;
 			case AUDIT_GID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_GID_TO_SGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_GID_TO_SGID;
 				break;
 			case AUDIT_OBJ_GID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_SGID_TO_OBJ_GID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_SGID_TO_OBJ_GID;
 				break;
 			case AUDIT_EGID:
-				rule->values[rule->field_count] = AUDIT_COMPARE_EGID_TO_SGID;
+				rule->values[rule->field_count] =
+						AUDIT_COMPARE_EGID_TO_SGID;
 				break;
 			default:
 				return -1;
@@ -1210,7 +1315,7 @@ int audit_determine_machine(const char *arch)
 #ifdef WITH_AARCH64
 		case MACH_AARCH64:
 			if (bits && bits != __AUDIT_ARCH_64BIT)
-				return -6;
+				return -6; // Deadcode - just incase of mistake
 			break;
 #endif
 		case MACH_86_64:   /* fallthrough */
@@ -1395,7 +1500,18 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 		case AUDIT_SUBJ_SEN:
 		case AUDIT_SUBJ_CLR:
 		case AUDIT_FILTERKEY:
-			if (field == AUDIT_FILTERKEY && !(_audit_syscalladded || _audit_permadded))
+		case AUDIT_EXE:
+			if (field == AUDIT_EXE) {
+				uint32_t features = audit_get_features();
+				if ((features & AUDIT_FEATURE_BITMAP_EXECUTABLE_PATH) == 0)
+					return -30;
+				if (op != AUDIT_EQUAL)
+					return -29;
+				_audit_exeadded = 1;
+			}
+			if (field == AUDIT_FILTERKEY &&
+				!(_audit_syscalladded || _audit_permadded ||
+				_audit_exeadded))
                                 return -19;
 			vlen = strlen(v);
 			if (field == AUDIT_FILTERKEY &&
@@ -1484,7 +1600,8 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 			}
 			break;
 		case AUDIT_FILETYPE:
-			if (!(flags == AUDIT_FILTER_EXIT || flags == AUDIT_FILTER_ENTRY))
+			if (!(flags == AUDIT_FILTER_EXIT ||
+						flags == AUDIT_FILTER_ENTRY))
 				return -17;
 			rule->values[rule->field_count] = 
 				audit_name_to_ftype(v);
@@ -1573,7 +1690,6 @@ int audit_detect_machine(void)
 		return audit_name_to_machine(uts.machine);
 	return -1;
 }
-hidden_def(audit_detect_machine)
 
 #ifndef NO_TABLES
 void audit_number_to_errmsg(int errnumber, const char *opt)
