@@ -1,5 +1,5 @@
 /* auditctl-listing.c -- 
- * Copyright 2014 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2014,16 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -63,7 +63,8 @@ int key_match(const struct audit_rule_data *r)
 		}
 		if (((field >= AUDIT_SUBJ_USER && field <= AUDIT_OBJ_LEV_HIGH)
                      && field != AUDIT_PPID) || field == AUDIT_WATCH ||
-			field == AUDIT_DIR || field == AUDIT_FILTERKEY) {
+			field == AUDIT_DIR || field == AUDIT_FILTERKEY
+		     || field == AUDIT_EXE) {
 				boffset += r->values[i];
 		}
 	}
@@ -350,13 +351,17 @@ static void print_rule(const struct audit_rule_data *r)
 				boffset += r->values[i];
 			} else if (field == AUDIT_DIR) {
 				if (watch)
-					printf("-w %.*s/", r->values[i],
+					printf("-w %.*s", r->values[i],
 						&r->buf[boffset]);
 				else
 					printf(" -F dir=%.*s", r->values[i],
 						&r->buf[boffset]);
 
 				boffset += r->values[i];
+			} else if (field == AUDIT_EXE) {
+				printf(" -F exe=%.*s",
+					r->values[i], &r->buf[boffset]);
+				boffset += r->values[i];
 			} else if (field == AUDIT_FILTERKEY) {
 				char *rkey, *ptr, *saved;
 				if (asprintf(&rkey, "%.*s", r->values[i],
@@ -471,7 +476,7 @@ const char *get_enable(unsigned e)
 		case 1:
 			return "enabled";
 		case 2:
-			return "enabl;ed+immutable";
+			return "enabled+immutable";
 		default:
 			return "unknown";
 	}
